@@ -9,6 +9,8 @@ from ...streams import (
 
 from .util import _trace_to_subplot
 
+from ...element import Tiles
+
 
 class PlotlyCallbackMetaClass(type):
     """
@@ -19,13 +21,13 @@ class PlotlyCallbackMetaClass(type):
     """
 
     def __init__(cls, name, bases, attrs):
-        super(PlotlyCallbackMetaClass, cls).__init__(name, bases, attrs)
+        super().__init__(name, bases, attrs)
 
         # Create weak-value dictionary to hold instances of the class
         cls.instances = WeakValueDictionary()
 
     def __call__(cls, *args, **kwargs):
-        inst = super(PlotlyCallbackMetaClass, cls).__call__(*args, **kwargs)
+        inst = super().__call__(*args, **kwargs)
 
         # Store weak reference to the callback instance in the _instances
         # WeakValueDictionary. This will allow instances to be garbage collected and
@@ -46,8 +48,10 @@ class PlotlyCallback(object):
         self.last_event = None
 
     @classmethod
-    def update_streams_from_property_update(cls, property_value, fig_dict):
-        event_data = cls.get_event_data_from_property_update(property_value, fig_dict)
+    def update_streams_from_property_update(cls, property, property_value, fig_dict):
+        event_data = cls.get_event_data_from_property_update(
+            property, property_value, fig_dict
+        )
         streams = []
         for trace_uid, stream_data in event_data.items():
             if trace_uid in cls.instances:
@@ -69,15 +73,15 @@ class PlotlyCallback(object):
             raise e
 
     @classmethod
-    def get_event_data_from_property_update(cls, property_value, fig_dict):
+    def get_event_data_from_property_update(cls, property, property_value, fig_dict):
         raise NotImplementedError
 
 
 class Selection1DCallback(PlotlyCallback):
-    callback_property = "selected_data"
+    callback_properties = ["selected_data"]
 
     @classmethod
-    def get_event_data_from_property_update(cls, selected_data, fig_dict):
+    def get_event_data_from_property_update(cls, property, selected_data, fig_dict):
 
         traces = fig_dict.get('data', [])
 
@@ -100,26 +104,38 @@ class Selection1DCallback(PlotlyCallback):
 
 
 class BoundsCallback(PlotlyCallback):
-    callback_property = "selected_data"
+    callback_properties = ["selected_data"]
     boundsx = False
     boundsy = False
 
     @classmethod
-    def get_event_data_from_property_update(cls, selected_data, fig_dict):
+    def get_event_data_from_property_update(cls, property, selected_data, fig_dict):
         traces = fig_dict.get('data', [])
 
-        if not selected_data or 'range' not in selected_data:
-            # No valid box selection
-            box = None
-        else:
-            # Get x and y axis references
-            box = selected_data["range"]
-            axis_refs = list(box)
-            xref = [ref for ref in axis_refs if ref.startswith('x')][0]
-            yref = [ref for ref in axis_refs if ref.startswith('y')][0]
-
-        # Process traces
+        # Initialize event data by clearing box selection on everything
         event_data = {}
+        for trace_ind, trace in enumerate(traces):
+            trace_uid = trace.get('uid', None)
+            if cls.boundsx and cls.boundsy:
+                stream_data = dict(bounds=None)
+            elif cls.boundsx:
+                stream_data = dict(boundsx=None)
+            elif cls.boundsy:
+                stream_data = dict(boundsy=None)
+            else:
+                stream_data = {}
+
+            event_data[trace_uid] = stream_data
+
+        range_data = (selected_data or {}).get("range", {})
+        cls.update_event_data_xyaxis(range_data, traces, event_data)
+        cls.update_event_data_mapbox(range_data, traces, event_data)
+
+        return event_data
+
+    @classmethod
+    def update_event_data_xyaxis(cls, range_data, traces, event_data):
+        # Process traces
         for trace_ind, trace in enumerate(traces):
             trace_type = trace.get('type', 'scatter')
             trace_uid = trace.get('uid', None)
@@ -127,10 +143,14 @@ class BoundsCallback(PlotlyCallback):
             if _trace_to_subplot.get(trace_type, None) != ['xaxis', 'yaxis']:
                 continue
 
-            if (box and trace.get('xaxis', 'x') == xref and
-                    trace.get('yaxis', 'y') == yref):
+            xref = trace.get('xaxis', 'x')
+            yref = trace.get('yaxis', 'y')
 
-                new_bounds = (box[xref][0], box[yref][0], box[xref][1], box[yref][1])
+            if xref in range_data and yref in range_data:
+                new_bounds = (
+                    range_data[xref][0], range_data[yref][0],
+                    range_data[xref][1], range_data[yref][1]
+                )
 
                 if cls.boundsx and cls.boundsy:
                     stream_data = dict(bounds=new_bounds)
@@ -139,23 +159,39 @@ class BoundsCallback(PlotlyCallback):
                 elif cls.boundsy:
                     stream_data = dict(boundsy=(new_bounds[1], new_bounds[3]))
                 else:
-                    stream_data = dict()
+                    stream_data = {}
 
                 event_data[trace_uid] = stream_data
-            else:
+
+    @classmethod
+    def update_event_data_mapbox(cls, range_data, traces, event_data):
+        # Process traces
+        for trace_ind, trace in enumerate(traces):
+            trace_type = trace.get('type', 'scatter')
+            trace_uid = trace.get('uid', None)
+
+            if _trace_to_subplot.get(trace_type, None) != ['mapbox']:
+                continue
+
+            mapbox_ref = trace.get('subplot', 'mapbox')
+            if mapbox_ref in range_data:
+                lon_bounds = [range_data[mapbox_ref][0][0], range_data[mapbox_ref][1][0]]
+                lat_bounds = [range_data[mapbox_ref][0][1], range_data[mapbox_ref][1][1]]
+
+                easting, northing = Tiles.lon_lat_to_easting_northing(lon_bounds, lat_bounds)
+                new_bounds = (easting[0], northing[0], easting[1], northing[1])
+
                 if cls.boundsx and cls.boundsy:
-                    stream_data = dict(bounds=None)
+                    stream_data = dict(bounds=new_bounds)
                 elif cls.boundsx:
-                    stream_data = dict(boundsx=None)
+                    stream_data = dict(boundsx=(new_bounds[0], new_bounds[2]))
                 elif cls.boundsy:
-                    stream_data = dict(boundsy=None)
+                    stream_data = dict(boundsy=(new_bounds[1], new_bounds[3]))
                 else:
-                    stream_data = dict()
+                    stream_data = {}
 
                 event_data[trace_uid] = stream_data
 
-        return event_data
-
 
 class BoundsXYCallback(BoundsCallback):
     boundsx = True
@@ -171,15 +207,23 @@ class BoundsYCallback(BoundsCallback):
 
 
 class RangeCallback(PlotlyCallback):
-    callback_property = "viewport"
+    callback_properties = ["viewport", "relayout_data"]
     x_range = False
     y_range = False
 
     @classmethod
-    def get_event_data_from_property_update(cls, viewport, fig_dict):
-
+    def get_event_data_from_property_update(cls, property, property_value, fig_dict):
         traces = fig_dict.get('data', [])
 
+        if property == "viewport":
+            event_data = cls.build_event_data_from_viewport(traces, property_value)
+        else:
+            event_data = cls.build_event_data_from_relayout_data(traces, property_value)
+
+        return event_data
+
+    @classmethod
+    def build_event_data_from_viewport(cls, traces, property_value):
         # Process traces
         event_data = {}
         for trace_ind, trace in enumerate(traces):
@@ -194,19 +238,65 @@ class RangeCallback(PlotlyCallback):
             xprop = '{xaxis}.range'.format(xaxis=xaxis)
             yprop = '{yaxis}.range'.format(yaxis=yaxis)
 
-            if not viewport:
+            if not property_value:
                 x_range = None
                 y_range = None
-            elif xprop in viewport and yprop in viewport:
-                x_range = tuple(viewport[xprop])
-                y_range = tuple(viewport[yprop])
-            elif xprop + "[0]" in viewport and xprop + "[1]" in viewport and \
-                    yprop + "[0]" in viewport and yprop + "[1]" in viewport :
-                x_range = (viewport[xprop + "[0]"], viewport[xprop + "[1]"])
-                y_range = (viewport[yprop + "[0]"], viewport[yprop + "[1]"])
+            elif xprop in property_value and yprop in property_value:
+                x_range = tuple(property_value[xprop])
+                y_range = tuple(property_value[yprop])
+            elif xprop + "[0]" in property_value and xprop + "[1]" in property_value and \
+                    yprop + "[0]" in property_value and yprop + "[1]" in property_value:
+                x_range = (property_value[xprop + "[0]"],property_value[xprop + "[1]"])
+                y_range = (property_value[yprop + "[0]"], property_value[yprop + "[1]"])
             else:
+                continue
+
+            stream_data = {}
+            if cls.x_range:
+                stream_data['x_range'] = x_range
+
+            if cls.y_range:
+                stream_data['y_range'] = y_range
+
+            event_data[trace_uid] = stream_data
+        return event_data
+
+    @classmethod
+    def build_event_data_from_relayout_data(cls, traces, property_value):
+        # Process traces
+        event_data = {}
+        for trace_ind, trace in enumerate(traces):
+            trace_type = trace.get('type', 'scattermapbox')
+            trace_uid = trace.get('uid', None)
+
+            if _trace_to_subplot.get(trace_type, None) != ['mapbox']:
+                continue
+
+            subplot_id = trace.get("subplot", "mapbox")
+            derived_prop = subplot_id + "._derived"
+
+            if not property_value:
                 x_range = None
                 y_range = None
+            elif "coordinates" in property_value.get(derived_prop, {}):
+                coords = property_value[derived_prop]["coordinates"]
+                ((lon_top_left, lat_top_left),
+                 (lon_top_right, lat_top_right),
+                 (lon_bottom_right, lat_bottom_right),
+                 (lon_bottom_left, lat_bottom_left)) = coords
+
+                lon_left = min(lon_top_left, lon_bottom_left)
+                lon_right = max(lon_top_right, lon_bottom_right)
+                lat_bottom = min(lat_bottom_left, lat_bottom_right)
+                lat_top = max(lat_top_left, lat_top_right)
+
+                x_range, y_range = Tiles.lon_lat_to_easting_northing(
+                    [lon_left, lon_right], [lat_bottom, lat_top]
+                )
+                x_range = tuple(x_range)
+                y_range = tuple(y_range)
+            else:
+                continue
 
             stream_data = {}
             if cls.x_range:
@@ -242,4 +332,3 @@ callbacks[BoundsY] = BoundsYCallback
 callbacks[RangeXY] = RangeXYCallback
 callbacks[RangeX] = RangeXCallback
 callbacks[RangeY] = RangeYCallback
-
